%get_image
% Open a bitmap image file and return a colour matrix (double precision, in
% range [0,1]) that can be used for surface texture mapping. The bitmap
% will be interpolated based upon the dimensions provided. If one or more
% of the dimensions is [] then the size of the colour matrix will be
% determined by the size of the bitmap.
%
% Syntax: I = get_image( width_pix, height_pix, fname )
%
% width_pix     Width of I in pixels.
% height_pix    Height of I in pixels.
% fname         I [] then a file browser box will open requesting a file.
%               If a genuine pathname to a file, this file will be loaded.
%
% I             R,G,B Colour matrix. Elements are floating point numbers from
%               range [0,1]. Dimensions are height_pix x width_pix x 3
%
% LAST UPDATED by Andy French 19/1/2014

function [I,fname] = get_image( width_pix, height_pix, fname )

% Choose bitmap file
if isempty(fname)
    [filename, pathname, filterindex] = uigetfile( {'*.jpg';'*.png';'*.bmp';'*.gif'},...
        'Choose bitmap image file') ;
    fname = [pathname,filename];
end

% Load bitmap file into array I. Convert it into a matrix of numbers
% ranging from 0 to 1.
try
    %Try and import image file
    [I,cmap] = imread(fname);
    
    %Convert image to double precision on [0,1]
    I = double(I)/255 ;
    I = flipdim(I,1);
    
    %Get dimensions of I
    [heightI,widthI,num_cols]  = size(I);
    
    %Take into account grayscale images, which are a single colour plane.
    %If the image is grayscale, repeat it for R,G and B planes.
    if num_cols==1
        II = zeros([heightI,widthI,3]);
        II(:,:,1) = I;
        II(:,:,2) = I;
        II(:,:,3) = I;
        I = II;
        clear II;
    end
    
    %Interpolate image if required
    if ( ~isempty( width_pix) ) && ( ~isempty( height_pix ) )
        
        %Define index vectors for height and width of 'initial_image'
        w = 1:widthI;
        h = 1:heightI;
        
        %Define index vectors for height and width of output image
        W = linspace(1,widthI,width_pix );
        H = linspace(1,heightI,height_pix );
        
        %Meshgrid these
        [h,w] = meshgrid(h,w);
        [H,W] = meshgrid(H,W);
        
        %Create R,G,B matricies from image matrix I.
        R = ones(heightI,widthI);
        G = ones(heightI,widthI);
        B = ones(heightI,widthI);
        R(:,:) = I(:,:,1);
        G(:,:) = I(:,:,2);
        B(:,:) = I(:,:,3);
        
        %Interpolate these colour matricies
        R = interp2(h,w,R.',H,W);
        G = interp2(h,w,G.',H,W);
        B = interp2(h,w,B.',H,W);
        clear h H w W
        
        %Initialise output array
        dim = size(R.');
        II = zeros(dim(1),dim(2),3);
        II(:,:,1) = R.';
        II(:,:,2) = G.';
        II(:,:,3) = B.';
        clear R G B
        
        %Assign output
        I = II;
        clear II;
    end
    
catch exception
    
    %Display file loading error to comand window
    disp(' ')
    %disp( exception )
    
    %If file doesn't exist or doesn't load, set random colours
    if ( isempty( width_pix) ) || ( isempty( height_pix ) )
        %Default image dimensions if no options set
        width_pix = 200;
        height_pix = 100;
    end
    I = rand( width_pix, height_pix, 3 );
end

%End of code