%spherium_lite
%
% Simple command line program which enables a surface defined by a radius
% which is a function of azimuth (a) and elevation (e). This can optionally
% be texture mapped and transparency mapped.
%
% LAST UPDATED by Andy French 16/07/2015

function spherium_lite

%Initialise global data store d which all sub-functions can read and
%write to
global d

%Set high resolution image dimensions
d.high_res_width_pix = 2000;
d.high_res_height_pix = 1000;

%

%% Command line interface %%

% Display welcome message to command line
disp(' ');
disp(' Welcome to SPHERIUM LITE ! ' )
disp(' By Andy French July 2015' )
disp(' ')
disp(' Action keys:')
disp(' q   Quit ')
disp(' t   Toggle transparency')
disp(' y   Load new transparency bitmap')
disp(' p   Print a JPEG image of current figure')
disp(' l   Toggle camlight')
disp(' b   Toggle bitmap texture')
disp(' n   Load new bitmap texture')
disp(' h   High resolution toggle')
disp(' s   Export .stl stereo-lithography file')
disp(' ')
disp(' Type radius of surface as a function of azimuth (a) and elevation (e) ...')

% Ask for mathematical function of azimuth and elevation which sets the
% radius of the surface
d.f = input( ' R = ','s' );

%Special test input!
if strcmp( d.f, '' )
    %Default inputs
    d.f = 'sin(a.*e)';
    disp(' ')
    disp( [' Default option selected ... R = ',d.f] );
    d.width_pix = 400;
    d.height_pix = 200;
    d.transparency_fname = [pwd,'\transparencies\holes.jpg'];
    d.texture_fname = [pwd,'\textures\shapes.jpg'];
    [d.B,d.texture_fname] = get_image( d.width_pix, d.height_pix, d.texture_fname );
    [d.T,d.transparency_fname] = get_image( d.width_pix, d.height_pix, d.transparency_fname );
else
    %Replace mathematical operations with a . to enable matrix elementwise
    %multiplication
    d.f = strrep(d.f,'*','.*');
    d.f = strrep(d.f,'/','./');
    d.f = strrep(d.f,'^','.^');
    
    %Request surface resolution, which will correspond to texture
    %and transparency bitmaps in pixels
    disp(' ')
    disp(' State number of azimuth steps')
    d.width_pix = str2num( input( ' width_pix = ','s' ));
    
    disp(' ')
    disp(' State number of elevation steps')
    d.height_pix = str2num( input( ' height_pix = ','s' ));
    
    %Select default texture bitmap
    disp(' ')
    disp(' Get default texture bitmap')
    [d.B,d.texture_fname] = get_image( d.width_pix, d.height_pix, [] );
    
    %Select default transparency bitmap
    disp(' ')
    disp(' Get default transparency bitmap')
    [d.T,d.transparency_fname] = get_image( d.width_pix, d.height_pix, [] );
end

%Store image height and width
d.orig_width_pix = d.width_pix;
d.orig_height_pix = d.height_pix;

%Set texture and transparency to be initially on
d.transparency = 1;
d.texture = 1;

%Set camlight to be initially on
d.light_effect = 1;

%Set high-resolution to be initially off
d.high_res = 0;

%Initialise surface handle
d.s = [];

%

%% Generate original surface %%

%Create figure
d.fig = figure('name','spherium lite',...
    'renderer','opengl','color',[1,1,1],'InvertHardCopy', 'off','KeyPressFcn',@keypressfunc);

%Generate surface
generate_surface;

%%

%Generate original surface
function generate_surface

%Allow read-write usage of global variable d
global d

%Start clock
t0 = tic;
disp(' ')
disp(' Generating surface ...')

%Create arrays of azimuth and elevation /radians
e = linspace( -pi/2, pi/2, d.height_pix );
a = linspace( -pi, pi, d.width_pix );

%Mesh azi and elev arrays
[a,e] = meshgrid(a,e);

%Determine radius using user-defined function f
eval( ['r = ',d.f,';'] );

%If radius is complex, make NaN which means it won't be plotted
r( imag(r) > 1e-6  ) = NaN;

%Make radius a positive number
%r = abs(r);

%Determine x,y,z matrices corresponding to surface
d.x = r.*cos(e).*cos(a);
d.y = r.*cos(e).*sin(a);
d.z = r.*sin(e);

%Plot surface, after deleting any previous surface
figure( d.fig );
delete(d.s);
d.s = plot_surface(d.x,d.y,d.z, d.texture, d.transparency, d.B, d.T, d.light_effect );

%Stop clock and display rendering time
disp(' ')
disp([ ' Surface generated in ',num2str(toc(t0)),' seconds.'] ) ;

%%

%Update figure - i.e. change is the transparency or texture, not the
%surface itself
function update_figure
global d
figure(d.fig);
[d.az,d.el] = view;
delete(d.s)
d.s = plot_surface(d.x,d.y,d.z, d.texture, d.transparency, d.B, d.T, d.light_effect );
view(d.az,d.el);

%%

%Plot surface
function s = plot_surface( x,y,z,texture,transparency, B, T, light_effect )

%Create surface object s
s = surf( x,y,z);
shading interp;
interp_colormap(max(size(x)));
view([110,36]);
axis vis3d
axis off
zoom(1.5);

%Set colours of surface points to match those of bitmap, i.e.
%texture-mapping the bitmap.
if texture ==1
    set( s, 'CDatamapping', 'Direct','CData', B );
end

%Set the transparency of the surface
if transparency == 1
    set( s, 'AlphaDataMapping','none','AlphaData', T,...
        'edgealpha','interp','facealpha','interp' );
end

%Add camlight
if light_effect == 1
    camlight;
end

%%

%Key press handler
function keypressfunc( fig,evnt )

%Allow read-write usage of global variable d
global d

%Get the string of the current key press
s = get( d.fig,'currentkey');

%Actions, depending on the key pressed...

if strcmp(s,'q') == 1
    %Quit
    close(d.fig)
    
elseif strcmp(s, 't') == 1
    %Toggle transparency
    if d.transparency==1
        d.transparency=0;
    else
        d.transparency=1;
    end
    update_figure
    
elseif strcmp(s, 'y') == 1
    %Load new transparency
    [d.T,d.transparency_fname] = get_image( d.width_pix, d.height_pix, [] );
    d.transparency=1;
    update_figure
    
elseif strcmp(s, 'b') == 1
    %Toggle textured surface
    if d.texture==1
        d.texture=0;
    else
        d.texture=1;
    end
    update_figure
    
elseif strcmp(s, 'n') == 1
    %Load new textured surface
    [d.B,d.texture_fname] = get_image( d.width_pix, d.height_pix, [] );
    d.texture = 1;
    update_figure
    
elseif strcmp(s, 'l') == 1
    %Toggle camlight
    if d.light_effect == 1
        d.light_effect = 0;
    else
        d.light_effect = 1;
    end
    update_figure;
    
elseif strcmp(s, 'h') == 1
    
    %Toggle high resolution
    if d.high_res == 1
        d.high_res = 0;
    else
        d.high_res = 1;
    end
    
    %Replace-surface with modified resolution (high-res or original)
    disp(' ')
    disp(' Toggling high resolution image, this may take a while....')
    t0 = tic;
    
    if d.high_res == 1
        %Replace image dimensions with high resolution version
        d.width_pix = d.high_res_width_pix;
        d.height_pix = d.high_res_height_pix;
    else
        %Replace image dimensions with original resolution version
        d.width_pix = d.orig_width_pix;
        d.height_pix = d.orig_height_pix;
    end
    
    %Reload texture and transpareny bitmaps
    [d.B,d.texture_fname] = get_image( d.width_pix, d.height_pix, d.texture_fname );
    [d.T,d.transparency_fname] = get_image( d.width_pix, d.height_pix, d.transparency_fname );
    
    %Re-generate surface at this new resolution
    [d.az,d.el] = view;
    generate_surface;
    
    %Set previous view
    view(d.az,d.el);
    
    %Display completion message to command window
    disp(' ')
    disp([ ' Surface re-generated in ',num2str(toc(t0)),' seconds.'] ) ;
    
elseif strcmp(s, 'p') == 1
    %Print figure
    print(d.fig,['spherium image ',strrep(datestr(now),':','-')],'-djpeg','-r600');
    
elseif strcmp(s, 'v') == 1
    %Display current view to command window
    disp(' ')
    [d.az,d.el] = view;
    disp([' Current [az,el] view is:   [',num2str(d.az),',',num2str(d.el),']' ] );
    
elseif strcmp(s, 's') == 1
    
    %If transparency applied, drill holes in the surface
    if d.transparency==1;
        
        %Set max value of transparency for holes to be 'drilled'
        Tmax = 0.1;
        
        %Combine transparency into a single matrix
        %Make sure it is only one 'page' i.e. greyscale. If not mean over the
        %other colours.
        [r,c,p] = size(d.T);
        if p~=1
            T = zeros(r,c);
            T = (1/3) * ( d.T(:,:,1)+d.T(:,:,2)+d.T(:,:,3) );
        else
            T=d.T;
        end
        
        %Store original x,y,z coordinates
        x = d.x;
        y = d.y;
        z = d.z;
 
        %Drill holes via setting x,y,z coordinates to be NaN
        d.x(T<Tmax) = NaN;
        d.y(T<Tmax) = NaN;
        d.z(T<Tmax) = NaN;
        clear T
        update_figure;
    end
    
    %Export .stl file of current surface
    stlwrite( ['spherium image ',strrep(datestr(now),':','-'),'.stl'],surf2patch(d.s,'triangles'));
    
    %Restore figure, if transparency has caused holes to be drilled
    if d.transparency==1;
        d.x = x;
        d.y = y;
        d.z = z;
        clear x y z
        update_figure;
    end
end

%End of code